@@ -1,11 +1,20 @@
-use std::{env, fs::File, io::Write, path::Path, process::Command, str};
-
 #[cfg(target_vendor = "apple")]
 use glob::glob;
-
 #[cfg(target_vendor = "apple")]
 use std::path::PathBuf;
+use std::{env, fs::File, io::Write, path::Path, process::Command, str};
+#[cfg(not(target_vendor = "apple"))]
+use which::which;
 
+/// The max version of `LLVM` we're looking for
+#[cfg(not(target_vendor = "apple"))]
+const LLVM_VERSION_MAX: u32 = 33;
+
+/// The min version of `LLVM` we're looking for
+#[cfg(not(target_vendor = "apple"))]
+const LLVM_VERSION_MIN: u32 = 6;
+
+/// Get the extension for a shared object
 fn dll_extension<'a>() -> &'a str {
     match env::var("CARGO_CFG_TARGET_OS").unwrap().as_str() {
         "windwos" => "dll",
@@ -53,6 +62,13 @@ fn find_llvm_config() -> String {
             }
         }
         #[cfg(not(target_vendor = "apple"))]
+        for version in (LLVM_VERSION_MIN..=LLVM_VERSION_MAX).rev() {
+            let llvm_config_name = format!("llvm-config-{}", version);
+            if which(&llvm_config_name).is_ok() {
+                return llvm_config_name;
+            }
+        }
+        #[cfg(not(target_vendor = "apple"))]
         "llvm-config".to_string()
     })
 }
@@ -62,6 +78,9 @@ fn main() {
     let out_dir = Path::new(&out_dir);
     let src_dir = Path::new("src");
 
+    println!("cargo:rerun-if-env-changed=LLVM_CONFIG");
+    println!("cargo:rerun-if-env-changed=LIBAFL_EDGES_MAP_SIZE");
+
     let mut custom_flags = vec![];
 
     let dest_path = Path::new(&out_dir).join("clang_constants.rs");
@@ -70,7 +89,6 @@ fn main() {
     let edges_map_size: usize = option_env!("LIBAFL_EDGES_MAP_SIZE")
         .map_or(Ok(65536), str::parse)
         .expect("Could not parse LIBAFL_EDGES_MAP_SIZE");
-    println!("cargo:rerun-if-env-changed=LIBAFL_EDGES_MAP_SIZE");
     custom_flags.push(format!("-DLIBAFL_EDGES_MAP_SIZE={}", edges_map_size));
 
     let llvm_config = find_llvm_config();
@@ -86,7 +104,9 @@ fn main() {
             &mut clang_constants_file,
             "// These constants are autogenerated by build.rs
 
+            /// The path to the `clang` executable
             pub const CLANG_PATH: &str = {:?};
+            /// The path to the `clang++` executable
             pub const CLANGXX_PATH: &str = {:?};
             
             /// The size of the edges map
@@ -147,15 +167,18 @@ fn main() {
             &mut clang_constants_file,
             "// These constants are autogenerated by build.rs
 
+/// The path to the `clang` executable
 pub const CLANG_PATH: &str = \"clang\";
+/// The path to the `clang++` executable
 pub const CLANGXX_PATH: &str = \"clang++\";
     "
         )
         .expect("Could not write file");
 
         println!(
-            "cargo:warning=Failed to locate the LLVM path using {}, we will not build LLVM passes",
-            llvm_config
+            "cargo:warning=Failed to locate the LLVM path using {}, we will not build LLVM passes
+            (if you need them, set point the LLVM_CONFIG env to a recent llvm-config, or make sure {} is available)",
+            llvm_config, llvm_config
         );
     }
 
