@@ -22,14 +22,19 @@ fn dll_extension<'a>() -> &'a str {
 
 include!(concat!(env!("OUT_DIR"), "/clang_constants.rs"));
 
+/// The supported LLVM passes
 #[allow(clippy::upper_case_acronyms)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum LLVMPasses {
     //CmpLogIns,
+    /// The CmpLog pass
     CmpLogRtn,
+    /// The AFL coverage pass
     AFLCoverage,
 }
 
 impl LLVMPasses {
+    /// Gets the path of the LLVM pass
     #[must_use]
     pub fn path(&self) -> PathBuf {
         match self {
@@ -43,6 +48,7 @@ impl LLVMPasses {
 
 /// Wrap Clang
 #[allow(clippy::struct_excessive_bools)]
+#[derive(Debug)]
 pub struct ClangWrapper {
     is_silent: bool,
     optimize: bool,
@@ -54,6 +60,8 @@ pub struct ClangWrapper {
     linking: bool,
     x_set: bool,
     bit_mode: u32,
+    need_libafl_arg: bool,
+    has_libafl_arg: bool,
 
     from_args_called: bool,
     base_args: Vec<String>,
@@ -104,6 +112,15 @@ impl CompilerWrapper for ClangWrapper {
 
         for arg in &args[1..] {
             match arg.as_ref() {
+                "--libafl-no-link" => {
+                    linking = false;
+                    self.has_libafl_arg = true;
+                    continue;
+                }
+                "--libafl" => {
+                    self.has_libafl_arg = true;
+                    continue;
+                }
                 "-x" => self.x_set = true,
                 "-m32" => self.bit_mode = 32,
                 "-m64" => self.bit_mode = 64,
@@ -197,6 +214,10 @@ impl CompilerWrapper for ClangWrapper {
             args.push(self.wrapped_cc.clone());
         }
         args.extend_from_slice(self.base_args.as_slice());
+        if self.need_libafl_arg && !self.has_libafl_arg {
+            return Ok(args);
+        }
+
         if !self.passes.is_empty() {
             args.push("-fno-experimental-new-pass-manager".into());
         }
@@ -260,6 +281,8 @@ impl ClangWrapper {
             linking: false,
             x_set: false,
             bit_mode: 0,
+            need_libafl_arg: false,
+            has_libafl_arg: false,
             from_args_called: false,
             base_args: vec![],
             cc_args: vec![],
@@ -269,11 +292,13 @@ impl ClangWrapper {
         }
     }
 
+    /// Sets the wrapped `cc` compiler
     pub fn wrapped_cc(&mut self, cc: String) -> &'_ mut Self {
         self.wrapped_cc = cc;
         self
     }
 
+    /// Sets the wrapped `cxx` compiler
     pub fn wrapped_cxx(&mut self, cxx: String) -> &'_ mut Self {
         self.wrapped_cxx = cxx;
         self
@@ -291,11 +316,23 @@ impl ClangWrapper {
         self
     }
 
-    // Add LLVM pass
+    /// Add LLVM pass
     pub fn add_pass(&mut self, pass: LLVMPasses) -> &'_ mut Self {
         self.passes.push(pass);
         self
     }
+
+    /// Set if linking
+    pub fn linking(&mut self, value: bool) -> &'_ mut Self {
+        self.linking = value;
+        self
+    }
+
+    /// Set if it needs the --libafl arg to add the custom arguments to clang
+    pub fn need_libafl_arg(&mut self, value: bool) -> &'_ mut Self {
+        self.need_libafl_arg = value;
+        self
+    }
 }
 
 #[cfg(test)]
